!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA GSFC Land Data Toolkit (LDT) V1.0
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
#include "LDT_misc.h"
module LDT_ANNMod
!BOP
!
!  !MODULE: LDT_ANNMod
! 
!  !DESCRIPTION: 
!   This implementation is based on the original code of 
!   Philip Brierly. The reference is: 
!   Bierly, P. and Batty, B., "Data mining with neural networks - 
!   an applied example in understanding electricity consumption
!   patterns", Chapter 12 in 'Knowledge discovery and data mining'
!   edited by M.A. Bramer, The Institution of Electrical Engineers, 
!   1999.
!   
!  !REVISION HISTORY: 
!  2 Oct 2008    Sujay Kumar  Initial Specification
! !USES: 

  use ESMF
  use LDT_coreMod
  use LDT_timeMgrMod
  use LDT_logMod
#if (defined USE_NETCDF3 || defined USE_NETCDF4) 
  use netcdf
#endif

  implicit none

  PRIVATE 
!-----------------------------------------------------------------------------
! !PUBLIC MEMBER FUNCTIONS: 
!-----------------------------------------------------------------------------
  public :: LDT_ANNinit
  public :: LDT_readANNinputData
  public :: LDT_readANNoutputData
  public :: LDT_tavgANNdata
  public :: LDT_logSingleANNdata
  public :: LDT_trainANN

!-----------------------------------------------------------------------------
! !PUBLIC TYPES:
!-----------------------------------------------------------------------------
  public :: LDT_ANNinput_data
  public :: LDT_ANNoutput_data
  public :: LDT_ANNdata

  type, public :: LDT_ANNmetadataEntry

     character*100, allocatable :: varName(:)
     character*50, allocatable  :: varUnits(:)

     real, allocatable     :: value(:,:,:) 

     integer, allocatable  :: count(:,:)
     real,    allocatable  :: value_hold(:,:)

  end type LDT_ANNmetadataEntry
  
  type, public :: ANNdatadec
     character*50           :: mode
     integer                :: nINPUTS
     integer                :: nOUTPUTS
     integer                :: nTOTAL
     integer                :: nppb
     integer                    :: nsources
     character*50, allocatable  :: inputsrc(:)
     integer,      allocatable  :: input_nparams(:)
     integer,      allocatable  :: input_param_s(:)
     integer,      allocatable  :: input_param_e(:)

     character*50           :: outputsrc
     integer                :: ntimes
     integer                :: hidden_neurons
     integer                :: nhs
     integer                :: nhf
     integer                :: nos
     real                   :: alr !learning rate
     real                   :: blr!learning rate
     integer                :: nIters
!     real, allocatable      :: trainingInputs(:,:,:)
!     real, allocatable      :: trainingOutputs(:,:)
!     real, allocatable      :: inputs_this_pattern(:,:)

     real                   :: rmse_best
     real, allocatable      :: wi(:,:,:)
     real, allocatable      :: wi_best(:,:,:)
     real, allocatable      :: wo(:,:)
     real, allocatable      :: wo_best(:,:)
     real, allocatable      :: hval(:)
     real, allocatable      :: dummy1(:,:,:)
     real, allocatable      :: dummy2(:,:,:)

     real, allocatable    :: maxinp(:,:)
     real, allocatable    :: mininp(:,:)
     real, allocatable    :: maxout(:,:)
     real, allocatable    :: minout(:,:)

     character*100             :: outfile
  end type ANNdatadec

  type(LDT_ANNmetadataEntry) :: LDT_ANNinput_data
  type(LDT_ANNmetadataEntry) :: LDT_ANNoutput_data

  type(ANNdatadec)          :: LDT_ANNdata

!EOP
  
contains
  subroutine LDT_ANNinit()
    
    integer :: rc
    integer                 :: nInputs
    integer                 :: nsize
    integer                 :: ntimes
    type(ESMF_Time)         :: startTime, stopTime
    type(ESMF_TimeInterval) :: timeStep
    integer                 :: n,i
    
    n = 1
    call ESMF_ConfigGetAttribute(LDT_config,LDT_ANNdata%mode,&
         label="ANN mode (training/validation):" ,rc=rc)
    call LDT_verify(rc,&
         'ANN mode (training/validation): option not specified in the config file')

    call ESMF_ConfigGetAttribute(LDT_config,LDT_ANNdata%nsources, &
         label="ANN number of input data sources:", rc=rc)
    call LDT_verify(rc,&
         'ANN number of input data sources: not specified')

    allocate(LDT_ANNdata%inputsrc(LDT_ANNdata%nsources))
    allocate(LDT_ANNdata%input_nparams(LDT_ANNdata%nsources))
    allocate(LDT_ANNdata%input_param_s(LDT_ANNdata%nsources))
    allocate(LDT_ANNdata%input_param_e(LDT_ANNdata%nsources))
    
    call ESMF_ConfigFindLabel(LDT_config,'ANN input data sources:',rc=rc)
    do i=1,LDT_ANNdata%nsources
       call ESMF_ConfigGetAttribute(LDT_config, LDT_ANNdata%inputsrc(i),rc=rc)
       call LDT_verify(rc,&
         'ANN input data sources: option not specified in the config file')
    enddo

    call ESMF_ConfigFindLabel(LDT_config,&
         'ANN number of parameters in each input source:',rc=rc)
    do i=1,LDT_ANNdata%nsources
       call ESMF_ConfigGetAttribute(LDT_config, &
            LDT_ANNdata%input_nparams(i),rc=rc)
       call LDT_verify(rc,&
         'ANN number of parameters in each input source: option not specified in the config file')
    enddo

    LDT_ANNdata%input_param_s(1) = 1
    LDT_ANNdata%input_param_e(1) = LDT_ANNdata%input_param_s(1) + & 
         LDT_ANNdata%input_nparams(1) - 1

    nInputs  = LDT_ANNdata%input_nparams(1)

    do i=2,LDT_ANNdata%nsources
       LDT_ANNdata%input_param_s(i) = LDT_ANNdata%input_param_e(i-1) + 1
       LDT_ANNdata%input_param_e(i) = LDT_ANNdata%input_param_s(i) + &
            LDT_ANNdata%input_nparams(i)-1
       
       nInputs = nInputs + LDT_ANNdata%input_nparams(i)
    enddo

    call ESMF_ConfigGetAttribute(LDT_config,LDT_ANNdata%outputsrc,&
         label="ANN output data source:" ,rc=rc)
    call LDT_verify(rc,&
         'ANN output data source: option not specified in the config file')

!    call ESMF_ConfigGetAttribute(LDT_config,nInputs,&
!         label="ANN number of input parameters:",rc=rc)
!    call LDT_verify(rc,&
!         'ANN number of input parameters: option not specified in the config file')

    call ESMF_ConfigGetAttribute(LDT_config,LDT_ANNdata%hidden_neurons,&
         label="ANN number of hidden neurons:",rc=rc)
    call LDT_verify(rc,&
         'ANN number of hidden neurons: option not specified in the config file')

    call ESMF_ConfigGetAttribute(LDT_config,LDT_ANNdata%nIters,&
         label="ANN number of iterations:",rc=rc)
    call LDT_verify(rc,&
         'ANN number of iterations: option not specified in the config file')
    
    call ESMF_ConfigGetAttribute(LDT_config,LDT_ANNdata%outfile,&
         label="ANN training output file:",rc=rc)
    call LDT_verify(rc,&
         'ANN training output file: option not specified in the config file')

    call ESMF_ClockGet(LDT_clock, startTime = startTime, &
         stopTime = stopTime)

    call ESMF_TimeIntervalSet(timeStep, s = nint(LDT_rc%tavgInterval),&
         rc=rc)

    ntimes = ((stopTime - startTime)/timestep)+1

    LDT_ANNdata%nInputs = nInputs
    LDT_ANNdata%ntimes = ntimes
    LDT_ANNdata%nOutputs = 1
    LDT_ANNdata%nTotal = LDT_ANNdata%nInputs + LDT_ANNdata%nOutputs
    LDT_ANNdata%nppb = LDT_ANNdata%nInputs + 1

    LDT_ANNdata%hidden_neurons = &
         LDT_ANNdata%hidden_neurons + 1 !acccounts for bias to output

!    LDT_ANNdata%nhs = LDT_ANNdata%nppb + 1
!    LDT_ANNdata%nhf = LDT_ANNdata%nppb + LDT_ANNdata%hidden_neurons
    LDT_ANNdata%nhs = 1
    LDT_ANNdata%nhf = LDT_ANNdata%hidden_neurons
    LDT_ANNdata%nos = LDT_ANNdata%nhf + 1
    
    LDT_ANNdata%alr = 0.1
    LDT_ANNdata%blr = LDT_ANNdata%alr/10.0

    allocate(LDT_ANNinput_data%value(&
         LDT_rc%ngrid(n),&
         LDT_ANNdata%nppb,&
         LDT_ANNdata%ntimes))
    allocate(LDT_ANNinput_data%value_hold(&
         LDT_rc%ngrid(n),&
         LDT_ANNdata%nInputs))
    allocate(LDT_ANNinput_data%count(&
         LDT_rc%ngrid(n),&
         LDT_ANNdata%nInputs))
    allocate(LDT_ANNinput_data%varName(&
         LDT_ANNdata%nppb))
    allocate(LDT_ANNinput_data%varUnits(&
         LDT_ANNdata%nppb))


    allocate(LDT_ANNoutput_data%value(&
         LDT_rc%ngrid(n),1,LDT_ANNdata%ntimes))
    allocate(LDT_ANNoutput_data%value_hold(&
         LDT_rc%ngrid(n),1))
    allocate(LDT_ANNoutput_data%count(&
         LDT_rc%ngrid(n),1))
    allocate(LDT_ANNoutput_data%varName(&
         1))
    allocate(LDT_ANNoutput_data%varUnits(&
         1))

    LDT_ANNinput_data%varName(LDT_ANNdata%nppb) = "Bias parameter"
    LDT_ANNinput_data%varUnits(LDT_ANNdata%nppb) = "-"

!    allocate(LDT_ANNdata%inputs_this_pattern(LDT_rc%ngrid(n),&
!         LDT_ANNdata%nppb))

!    LDT_ANNdata%inputs_this_pattern = 0 

    LDT_ANNinput_data%value = LDT_rc%udef
    LDT_ANNinput_data%value(:,LDT_ANNdata%nppb,:) = 1
    LDT_ANNinput_data%value_hold = 0 
    LDT_ANNinput_data%count = 0 
    
    LDT_ANNoutput_data%value = LDT_rc%udef
    LDT_ANNoutput_data%value_hold = 0 
    LDT_ANNoutput_data%count = 0 

    allocate(LDT_ANNdata%maxinp(LDT_rc%ngrid(n),LDT_ANNdata%nppb))
    allocate(LDT_ANNdata%mininp(LDT_rc%ngrid(n),LDT_ANNdata%nppb))
    allocate(LDT_ANNdata%maxout(LDT_rc%ngrid(n),1))
    allocate(LDT_ANNdata%minout(LDT_rc%ngrid(n),1))

    do i=1,LDT_ANNdata%nsources
       call setupANNinputsource(trim(LDT_ANNdata%inputsrc(i))//char(0))
    enddo
    call setupANNoutputsource(trim(LDT_ANNdata%outputsrc)//char(0))

  end subroutine LDT_ANNinit

  subroutine LDT_readANNinputData(n)

    integer, intent(in) :: n 
    integer             :: iomode
    integer             :: i

    iomode = 1
    do i=1,LDT_ANNdata%nsources
       call readANNinputsource(trim(LDT_ANNdata%inputsrc(i))//char(0),&
            n,iomode,LDT_ANNdata%input_param_s(i),LDT_ANNdata%input_param_e(i))
    enddo

  end subroutine LDT_readANNinputData

  subroutine LDT_readANNoutputData(n)

    integer, intent(in) :: n 
    integer             :: iomode

    iomode = 2
    call readANNoutputsource(trim(LDT_ANNdata%outputsrc)//char(0),&
         n,iomode,1,1)

  end subroutine LDT_readANNoutputData

  subroutine LDT_tavgANNdata(n)
    integer, intent(in) :: n 

    type(ESMF_Time)         :: currTime, startTime
    type(ESMF_TimeInterval) :: timeStep
    integer                 :: tindex
    integer                 :: c,r,pindex, gid
    integer                 :: rc, status

    if(mod(real(LDT_rc%hr)*3600+60*real(LDT_rc%mn)+float(LDT_rc%ss),&
            real(LDT_rc%tavgInterval)).eq.0) then        
       
       call ESMF_TimeSet(currTime, yy=LDT_rc%yr, &
            mm=LDT_rc%mo,dd=LDT_rc%da,h=LDT_rc%hr,&
            m=LDT_rc%mn,s=LDT_rc%ss,calendar=LDT_calendar,&
            rc=status)
       call LDT_verify(status, 'error in ESMF_TimeSet: in LDT_ANNMod')

       call ESMF_ClockGet(LDT_clock, startTime = startTime)
       
       call ESMF_TimeIntervalSet(timeStep, s = nint(LDT_rc%tavgInterval),&
            rc=rc)

       tindex = ((currTime - startTime)/timestep)+1
       
       do r=1,LDT_rc%lnr(n)
          do c=1,LDT_rc%lnc(n)
             do pindex=1,LDT_ANNdata%nInputs
                if(LDT_domain(n)%gindex(c,r).ne.-1) then 
                   gid = LDT_domain(n)%gindex(c,r)
                   if(LDT_ANNinput_data%count(gid,pindex).gt.0) then 
                      LDT_ANNinput_data%value_hold(gid,pindex) = & 
                           LDT_ANNinput_data%value_hold(gid,pindex) /& 
                           LDT_ANNinput_data%count(gid,pindex) 
                      LDT_ANNinput_data%value(gid,pindex,tindex) = &
                           LDT_ANNinput_data%value_hold(gid,pindex)
                      
                      LDT_ANNinput_data%value_hold(gid,pindex) = 0
                      LDT_ANNinput_data%count(gid,pindex) = 0 
                   endif
                endif
             enddo
          enddo
       enddo

       do r=1,LDT_rc%lnr(n)
          do c=1,LDT_rc%lnc(n)
             if(LDT_domain(n)%gindex(c,r).ne.-1) then 
                gid = LDT_domain(n)%gindex(c,r)
                if(LDT_ANNoutput_data%count(gid,1).gt.0) then 
                   LDT_ANNoutput_data%value_hold(gid,1) = & 
                        LDT_ANNoutput_data%value_hold(gid,1) / & 
                        LDT_ANNoutput_data%count(gid,1)
                   LDT_ANNoutput_data%value(gid,1,tindex) = &
                        LDT_ANNoutput_data%value_hold(gid,1) 
                   
                   LDT_ANNoutput_data%value_hold(gid,1) = 0
                   LDT_ANNoutput_data%count(gid,1) = 0 
                endif
             endif
          enddo
       enddo
    endif
  end subroutine LDT_tavgANNdata

  subroutine LDT_trainANN(n)

    integer, intent(in) :: n 

    integer             :: i,j,t
    integer             :: iPAT_NUM
    real                :: rRand
    real                :: output_this_pat
    real                :: er_this_pat
    real                :: rmse
    real                :: dummy2(LDT_ANNdata%nppb,1)
    real                :: dummy1(1,LDT_ANNdata%nhf)
    real                :: inputs_this_pattern(LDT_ANNdata%nppb)

    if(LDT_rc%endtime.eq.1) then 

          
       call random_seed      
       call ANN_scaleData(n)       
       call ANN_initiateWeights(n)

       if(LDT_ANNdata%mode.eq."training") then 
          write(LDT_logunit,*) 'Completed reading datasets...'
          
          do t = 1, LDT_rc%ngrid(n)
             do j=1,LDT_ANNdata%nIters
                do i=1,LDT_ANNdata%ntimes
                   call random_number(rRand)

                   !select a pattern at random
                   iPAT_NUM= nint(rRand*(LDT_ANNdata%ntimes-1))+1
                   !select the data to this pattern
!                   iPAT_NUM = i
                   inputs_this_pattern = & 
                        LDT_ANNinput_data%value(t,:,iPAT_NUM)
                   output_this_pat = LDT_ANNoutput_data%value(t,1,iPAT_NUM)

                   er_this_pat = calc_err_pat(t,&
                        inputs_this_pattern,&
                        output_this_pat, & 
                        LDT_ANNdata%wi(t,:,:), & 
                        LDT_ANNdata%wo(t,:))

                   !                print*, 'ip',inputs_this_pattern
                   !                print*, 'ou ',output_this_pat
                   !                print*, 'wi ',LDT_ANNdata%wi(t,:,:)
                   !                print*, 'wo ',LDT_ANNdata%wo(t,:)
                   !                print*, 'er ',er_this_pat

                   LDT_ANNdata%wo(t,:) = LDT_ANNdata%wo(t,:) - &
                        LDT_ANNdata%blr*&
                        LDT_ANNdata%hval(:)*&
                        er_this_pat
                   dummy2(:,1) = LDT_ANNinput_data%value(t,:,iPAT_NUM)
                   dummy1(1,:) = er_this_pat*LDT_ANNdata%wo(t,:)*&
                        (1-(LDT_ANNdata%hval(:)**2))
                   LDT_ANNdata%wi(t,:,:)  = & 
                        LDT_ANNdata%wi(t,:,:) - & 
                        (matmul(dummy2,dummy1)*LDT_ANNdata%alr)
                enddo
                rmse = calculate_error(t)

                call keep_best_weights(t,j,rmse)
                if(t.eq.1) write(LDT_logunit,*)  j, LDT_ANNdata%rmse_best
             enddo
          enddo

          call writeANNoutput(n)

       elseif(LDT_ANNdata%mode.eq."validation") then 
          write(LDT_logunit,*) 'Reading ANN output..'
          call readANNoutput(n)
          write(LDT_logunit,*) 'Done reading ANN output..'
          do i=1,LDT_ANNdata%ntimes
             do t=1,LDT_rc%ngrid(n)
                call calc_out_pat(t, &
                     LDT_ANNinput_data%value(t,:,i), &
                     output_this_pat, &
                     LDT_ANNdata%wi(t,:,:),&
                     LDT_ANNdata%wo(t,:))
!                if(t.eq.1) print*, &
!                     (LDT_ANNinput_data%value(t,1,i)/2.0+0.5)* &
!                     (LDT_ANNdata%maxinp(t,1)-LDT_ANNdata%mininp(t,1)),&
!                     (output_this_pat/2.0 + 0.5) * & 
!                     (LDT_ANNdata%maxout(t,1)-LDT_ANNdata%minout(t,1)) + & 
!                     LDT_ANNdata%minout(t,1)                     
                if(t.eq.1) print*, &
                     (output_this_pat/2.0 + 0.5) * & 
                     (LDT_ANNdata%maxout(t,1)-LDT_ANNdata%minout(t,1)) + & 
                     LDT_ANNdata%minout(t,1)                     
             enddo
          enddo
       endif
    endif
  end subroutine LDT_trainANN

  subroutine readANNoutput(n)

    integer, intent(in)   :: n 

    integer           :: iret
    integer           :: ftn
    integer           :: dimID(3)
    integer           :: tdimID(2)
    integer           :: wiID, woID
    integer           :: minrangeoutId, maxrangeoutId
    integer           :: minrangeinpId, maxrangeinpId
    character(len=8)  :: date
    character(len=10) :: time
    character(len=5)  :: zone
    integer, dimension(8) :: values
    character*100         :: outfile

#if (defined USE_NETCDF3 || defined USE_NETCDF4) 
    write(LDT_logunit,*) "Reading from LDT output directory: ",LDT_rc%odir
    outfile = trim(LDT_rc%odir)//'/'//&
         trim(LDT_ANNdata%outfile)         
    if(LDT_masterproc) then 
       iret = nf90_open(path=trim(outfile),&
            mode=NF90_NOWRITE, ncid=ftn)
       call LDT_verify(iret,'Error opening file '//trim(outfile))
    endif

    call LDT_verify(nf90_inq_varid(ftn,&
         'MINRANGE_INPUTS',&
         varid = minrangeinpId),&
         'nf90_inq_varid failed for MINRANGE_INPUTS')

    call LDT_verify(nf90_inq_varid(ftn,&
         'MAXRANGE_INPUTS',&
         varid = maxrangeinpId),&
         'nf90_inq_varid failed for MAXRANGE_INPUTS')

    call LDT_verify(nf90_inq_varid(ftn,&
         'MINRANGE_OUTPUTS',&
         varid = minrangeoutId),&
         'nf90_inq_varid failed for MINRANGE_OUTPUTS')

    call LDT_verify(nf90_inq_varid(ftn,&
         'MAXRANGE_OUTPUTS',&
         varid = maxrangeoutId),&
         'nf90_inq_varid failed for MAXRANGE_OUTPUTS')

    call LDT_verify(nf90_inq_varid(ftn,&
         'WI',&
         varid = wiId),&
         'nf90_inq_varid failed for WI')

    call LDT_verify(nf90_inq_varid(ftn,&
         'WO',&
         varid = woId),&
         'nf90_inq_varid failed for WO')
    
    call LDT_verify(nf90_get_var(ftn,&
         maxrangeinpId, & 
         LDT_ANNdata%maxinp),&
         'nf90_get_var failed for MAXRANGE_INPUTS')

    call LDT_verify(nf90_get_var(ftn,&
         minrangeinpId, & 
         LDT_ANNdata%mininp),&
         'nf90_get_var failed for MINRANGE_INPUTS')

    call LDT_verify(nf90_get_var(ftn,&
         maxrangeoutId, & 
         LDT_ANNdata%maxout(:,1)),&
         'nf90_get_var failed for MAXRANGE_OUTPUTS')

    call LDT_verify(nf90_get_var(ftn,&
         minrangeoutId, & 
         LDT_ANNdata%minout(:,1)),&
         'nf90_get_var failed for MINRANGE_OUTPUTS')

    call LDT_verify(nf90_get_var(ftn,&
         wiId, & 
         LDT_ANNdata%wi_best),&
         'nf90_get_var failed for WI')

    call LDT_verify(nf90_get_var(ftn,&
         woId, & 
         LDT_ANNdata%wo_best),&
         'nf90_get_var failed for WO')

    if(LDT_masterproc) then 
       iret = nf90_close(ftn)
       call LDT_verify(iret,'Error in nf90_close')
    endif
    LDT_ANNdata%wi = LDT_ANNdata%wi_best
    LDT_ANNdata%wo = LDT_ANNdata%wo_best
#endif    
  end subroutine readANNoutput

  subroutine writeANNoutput(n)

    integer,   intent(in) :: n 

    integer               :: iret
    integer               :: ftn
    integer               :: dimID(3)
    integer               :: tdimID(2)
    integer               :: wiID, woID
    integer               :: maxrangeOutID, minrangeOutID
    integer               :: maxrangeInpID, minrangeInpID
    integer               :: inputVarId, inputUnitId
    integer               :: i

    character(len=8)      :: date
    character(len=10)     :: time
    character(len=5)      :: zone
    integer, dimension(8) :: values
    character*100         :: outfile
    character*1000        :: allVarNames
    character*1000        :: allUnitNames

#if (defined USE_NETCDF3 || defined USE_NETCDF4) 
    call system('mkdir -p '//(LDT_rc%odir))
    write(LDT_logunit,*) "Writing to LDT output directory: ",&
          trim(LDT_rc%odir)
    
    outfile = trim(LDT_rc%odir)//'/'//&
         trim(LDT_ANNdata%outfile)         
    if(LDT_masterproc) then 
#if (defined USE_NETCDF3)
       iret = nf90_create(path=trim(outfile),&
            mode=NF90_CLOBBER, ncid=ftn)
#endif
#if (defined USE_NETCDF4)
       iret = nf90_create(path=trim(outfile),&
            cmode=NF90_NETCDF4, ncid=ftn)
#endif
       call LDT_verify(iret,'Error opening file '//trim(outfile))
    endif

    call date_and_time(date,time,zone,values)
    call LDT_verify(nf90_def_dim(ftn,'ngrid',&
         LDT_rc%glbngrid(n),dimID(1)))
    call LDT_verify(nf90_def_dim(ftn,'nppb',&
         LDT_ANNdata%nppb,dimID(2)))
    call LDT_verify(nf90_def_dim(ftn,'nhsize',&
         LDT_ANNdata%nhf,dimID(3)))

    tdimID(1) = dimID(1)
    tdimID(2) = dimID(3)
    
    call LDT_verify(nf90_put_att(ftn,NF90_GLOBAL,&
         "Activation_function", "Continuous Tan-Sigmoid"))          
    allVarNames = ""
    allUnitNames = ""
    do i=1,LDT_ANNdata%nppb-1
       if(i.eq.LDT_ANNdata%nppb-1) then 
          allVarnames = trim(allVarNames)//&
               trim(LDT_ANNinput_data%varName(i))
          allUnitnames = trim(allUnitNames)//&
               trim(LDT_ANNinput_data%varUnits(i))
       else
          allVarnames = trim(allVarNames)//&
               trim(LDT_ANNinput_data%varName(i))//","
          allUnitnames = trim(allUnitNames)//&
               trim(LDT_ANNinput_data%varUnits(i))//","
       endif
    enddo
    call LDT_verify(nf90_put_att(ftn,NF90_GLOBAL,&
         "Input_variables",&
         trim(allVarNames)))
    call LDT_verify(nf90_put_att(ftn,NF90_GLOBAL,&
         "Input_variable_units ",&
         trim(allUnitNames)))
    call LDT_verify(nf90_put_att(ftn,NF90_GLOBAL,&
         "missing_value", -9999.0))          
    call LDT_verify(nf90_put_att(ftn,NF90_GLOBAL,&
         "title", &
         "Land Data Toolkit (LDT) output"))
    call LDT_verify(nf90_put_att(ftn,NF90_GLOBAL,&
         "institution", &
         "NASA GSFC Hydrological Sciences Laboratory"))
    call LDT_verify(nf90_put_att(ftn,NF90_GLOBAL,&
         "history", &
         "created on date: "//date(1:4)//"-"//date(5:6)//"-"//&
         date(7:8)//"T"//time(1:2)//":"//time(3:4)//":"//time(5:10)))
    call LDT_verify(nf90_put_att(ftn,NF90_GLOBAL,"references", &
         "Kumar_etal_EMS_2006, Peters-Lidard_etal_ISSE_2007"))
    call LDT_verify(nf90_put_att(ftn,NF90_GLOBAL,"comment", &
         "website: http://lis.gsfc.nasa.gov/"))

    call LDT_verify(nf90_def_var(ftn,&
         'MINRANGE_INPUTS',&
         nf90_float, & 
         dimIDs = dimID(1:2),&
         varid = minrangeInpId),&
         'nf90_def_var failed for MINRANGE_INPUTS')

    call LDT_verify(nf90_def_var(ftn,&
         'MAXRANGE_INPUTS',&
         nf90_float, & 
         dimIDs = dimID(1:2),&
         varid = maxrangeInpId),&
         'nf90_def_var failed for MAXRANGE_INPUTS')

    call LDT_verify(nf90_def_var(ftn,&
         'MINRANGE_OUTPUTS',&
         nf90_float, & 
         dimIDs = dimID(1),&
         varid = minrangeOutId),&
         'nf90_def_var failed for MINRANGE_OUTPUTS')

    call LDT_verify(nf90_def_var(ftn,&
         'MAXRANGE_OUTPUTS',&
         nf90_float, & 
         dimIDs = dimID(1),&
         varid = maxrangeOutId),&
         'nf90_def_var failed for MAXRANGE_OUTPUTS')

    call LDT_verify(nf90_def_var(ftn,&
         'WI',&
         nf90_float, & 
         dimIDs = dimID,&
         varid = wiId),&
         'nf90_def_var failed for WI')

    call LDT_verify(nf90_def_var(ftn,&
         'WO',&
         nf90_float, & 
         dimIDs = tdimID,&
         varid = woId),&
         'nf90_def_var failed for WO')
    
    call LDT_verify(nf90_enddef(ftn))
    
    call LDT_verify(nf90_put_var(ftn,&
         maxrangeInpId, & 
         LDT_ANNdata%maxinp,&
         (/1,1/),&
         (/LDT_rc%glbngrid(n),LDT_ANNdata%nppb/)), &
         'nf90_put_var failed for MAXRANGE_INPUTS')

    call LDT_verify(nf90_put_var(ftn,&
         minrangeInpId, & 
         LDT_ANNdata%mininp,&
         (/1,1/),&
         (/LDT_rc%glbngrid(n),LDT_ANNdata%nppb/)), &
         'nf90_put_var failed for MINRANGE_INPUTS')

    call LDT_verify(nf90_put_var(ftn,&
         maxrangeOutId, & 
         LDT_ANNdata%maxout(:,1),&
         (/1/),&
         (/LDT_rc%glbngrid(n)/)), &
         'nf90_put_var failed for MAXRANGE_OUTPUTS')

    call LDT_verify(nf90_put_var(ftn,&
         minrangeOutId, & 
         LDT_ANNdata%minout(:,1),&
         (/1/),&
         (/LDT_rc%glbngrid(n)/)), &
         'nf90_put_var failed for MINRANGE_OUTPUTS')

    call LDT_verify(nf90_put_var(ftn,&
         wiId, & 
         LDT_ANNdata%wi_best,&
         (/1,1,1/),&
         (/LDT_rc%glbngrid(n),LDT_ANNdata%nppb,&
         LDT_ANNdata%nhf/)), &
         'nf90_put_var failed for WI')

    call LDT_verify(nf90_put_var(ftn,&
         woId, & 
         LDT_ANNdata%wo_best,&
         (/1,1/),&
         (/LDT_rc%glbngrid(n),&
         LDT_ANNdata%nhf/)), &
         'nf90_put_var failed for WO')

#if (defined USE_NETCDF3 || defined USE_NETCDF4)    
    if(LDT_masterproc) then 
       iret = nf90_close(ftn)
       call LDT_verify(iret,'Error in nf90_close')
    endif
#endif
#endif

  end subroutine writeANNoutput


  subroutine keep_best_weights(t,j,rmse)
    
    integer,   intent(in)       :: t
    integer,   intent(in)       :: j
    real                        :: rmse

    if(j.eq.1) then 
       LDT_ANNdata%rmse_best = rmse
    endif
    
    if(rmse < LDT_ANNdata%rmse_best) then 
       LDT_ANNdata%wi_best(t,:,:) = LDT_ANNdata%wi(t,:,:)
       LDT_ANNdata%wo_best(t,:) = LDT_ANNdata%wo(t,:)
       LDT_ANNdata%rmse_best = rmse
    else
       LDT_ANNdata%wi(t,:,:) = LDT_ANNdata%wi_best(t,:,:)
       LDT_ANNdata%wo(t,:) = LDT_ANNdata%wo_best(t,:)
    endif

  end subroutine keep_best_weights

  real function calculate_error(t)
    
    integer, intent(in) :: t
    
    real                :: sqErr
    integer             :: i
    real                :: err_this_pat, output_this_pat
    real                :: inputs_this_pat(LDT_ANNdata%nppb)

    sqErr = 0.0

    do i=1, LDT_ANNdata%ntimes
       
       output_this_pat = LDT_ANNoutput_data%value(t,1,i)
       inputs_this_pat(:) = LDT_ANNinput_data%value(t,:,i)
       err_this_pat = calc_err_pat(t,&
            inputs_this_pat, & 
            output_this_pat, & 
            LDT_ANNdata%wi(t,:,:), & 
            LDT_ANNdata%wo(t,:))

       sqErr = sqErr + err_this_pat**2
    enddo
    
    calculate_error = sqrt(sqErr/LDT_ANNdata%ntimes)
    
  end function calculate_error


  real function calc_err_pat(t, inputs_tp, output_tp, wi, wo)
    integer                 :: t
    real,        intent(in) :: inputs_tp(LDT_ANNdata%nppb)
    real,        intent(in) :: output_tp
    real                    :: wi(LDT_ANNdata%nppb, &
         LDT_ANNdata%nhf)
    real                    :: wo(LDT_ANNdata%nhf)
    real         :: outpredl
    
    LDT_ANNdata%hval(:) = tanh(matmul(transpose(&
         wi),inputs_tp))
    LDT_ANNdata%hval(LDT_ANNdata%nhf) = 1
    
    outpredl = sum(wo*LDT_ANNdata%hval(:))
!    print*,output_tp,& 
!         (outpredl/2.0 + 0.5) * & 
!         (LDT_ANNdata%maxout(t,1)-LDT_ANNdata%minout(t,1)) + & 
!         LDT_ANNdata%minout(t,1)        
    calc_err_pat = (outpredl-output_tp)

  end function calc_err_pat

  subroutine calc_out_pat(t, inputs_tp, output_tp, wi, wo)
    integer                 :: t
    real                    :: inputs_tp(LDT_ANNdata%nppb)
    real                    :: output_tp
    real                    :: wi(LDT_ANNdata%nppb, &
         LDT_ANNdata%nhf)
    real                    :: wo(LDT_ANNdata%nhf)
!    real                    :: inp_tmp(1,LDT_ANNdata%nppb)
!    real                    :: out_tmp(1,1)
!    real                    :: wo_tmp(LDT_ANNdata%nhf,1)

!    inp_tmp(1,:) = inputs_tp(:)
!    wo_tmp(:,1) = wo(:)

!    out_tmp= matmul(tanh(matmul(transpose(inp_tmp),wi)),wo_tmp)
!    output_tp = out_tmp(1,1)
    LDT_ANNdata%hval(:) = tanh(matmul(transpose(&
         wi),inputs_tp))
    LDT_ANNdata%hval(LDT_ANNdata%nhf) = 1
    
    output_tp = sum(wo*LDT_ANNdata%hval(:))

  end subroutine calc_out_pat

  subroutine ANN_initiateWeights(n)
    integer, intent(in) :: n 

    integer             :: t,j,k
    real                :: rRand

    allocate(LDT_ANNdata%wi(LDT_rc%ngrid(n),&
         LDT_ANNdata%nppb, &
         LDT_ANNdata%nhf))

    allocate(LDT_ANNdata%wi_best(LDT_rc%ngrid(n),&
         LDT_ANNdata%nppb, &
         LDT_ANNdata%nhf))

    allocate(LDT_ANNdata%wo(LDT_rc%ngrid(n),&
         LDT_ANNdata%nhf))

    allocate(LDT_ANNdata%wo_best(LDT_rc%ngrid(n),&
         LDT_ANNdata%nhf))

    allocate(LDT_ANNdata%hval(&
         LDT_ANNdata%nhf))

    do t =1, LDT_rc%ngrid(n)
       do k=1, LDT_ANNdata%nhf
          call random_number(rRand)
          LDT_ANNdata%wo(t,k) = ((rRand-0.5)*2)/10
          do j=1,LDT_ANNdata%nppb
             call random_number(rRand)
             LDT_ANNdata%wi(t,j,k) = ((rRand-0.5)*2)/10
          enddo
       enddo
    enddo
    LDT_ANNdata%wi_best = LDT_ANNdata%wi
    LDT_ANNdata%wo_best = LDT_ANNdata%wo

  end subroutine ANN_initiateWeights

#if 0 
  subroutine ANN_createTrainingData(n)
    
    integer, intent(in) :: n 

    rTRPC = 1.0
    rTEPC = rTRPC

    trainCount = 0 

    allocate(LDT_ANNdata%trainingInputs(LDT_rc%ngrid(n),&
         LDT_ANNdata%ntimes, &
         LDT_ANNdata%nppb))
    allocate(LDT_ANNdata%trainingOutputs(LDT_rc%ngrid(n),&
         LDT_ANNdata%ntimes))
    allocate(LDT_ANNdata%inputs_this_pattern(LDT_rc%ngrid(n),&
         LDT_ANNdata%nppb))

    do i=1,LDT_ANNdata%ntimes
       
       call random_number(rRand)
       if((rRand <= rTRPC).and.(trainCount < LDT_ANNdata%ntimes)) then 
          trainCount = trainCount + 1

          LDT_ANNdata%trainingInputs(:,trainCount,1:LDT_ANNdata%nInputs) = & 
               LDT_ANNinput_data%value(:,1:LDT_ANNdata%nInputs,i)
          LDT_ANNdata%trainingInputs(:,trainCount,LDT_ANNdata%nppb) = 1 
          LDT_ANNdata%trainingOutputs(:,trainCount) = &
               LDT_ANNoutput_data%value(:,1,i)
       elseif(trainCount < LDT_ANNdata%ntimes) then 
          trainCount = trainCount + 1
          LDT_ANNdata%trainingInputs(:,trainCount,1:LDT_ANNdata%nInputs) = & 
               LDT_ANNinput_data%value(:,1:LDT_ANNdata%nInputs,i)
          LDT_ANNdata%trainingInputs(:,trainCount,LDT_ANNdata%nppb) = 1 
          LDT_ANNdata%trainingOutputs(:,trainCount) = &
               LDT_ANNoutput_data%value(:,1,i)

       endif
    enddo

  end subroutine ANN_createTrainingData
#endif

  subroutine ANN_scaleData(n)

    integer, intent(in) :: n 

    integer :: t,p,g,count
    real    :: temp_in(LDT_rc%ngrid(n),LDT_ANNdata%nppb, LDT_ANNdata%ntimes)
    real    :: temp_out(LDT_rc%ngrid(n), 1, LDT_ANNdata%ntimes)
    logical :: check

    count = 0
    do t=1,LDT_ANNdata%ntimes
       check = .true. 
       do p=1,LDT_ANNdata%nppb
          do g=1,LDT_rc%ngrid(n)
             check = check .and. & 
                  (LDT_ANNinput_data%value(g,p,t).ne.LDT_rc%udef).and.&
                  (LDT_ANNoutput_data%value(g,1,t).ne.LDT_rc%udef)
             
          enddo
       enddo
       if(check) then 
          count = count + 1
          temp_in(:,:,count) = LDT_ANNinput_data%value(:,:,t)
          temp_out(:,:,count) = LDT_ANNoutput_data%value(:,:,t)
       endif
    enddo
!    stop
    LDT_ANNdata%ntimes = count
    deallocate(LDT_ANNinput_data%value)
    deallocate(LDT_ANNoutput_data%value)

    allocate(LDT_ANNinput_data%value(LDT_rc%ngrid(n),&
         LDT_ANNdata%nppb, &
         LDT_ANNdata%ntimes))

    allocate(LDT_ANNoutput_data%value(LDT_rc%ngrid(n),&
         1, &
         LDT_ANNdata%ntimes))

    do t=1, LDT_ANNdata%ntimes       
       LDT_ANNinput_data%value(:,:,t) = temp_in(:,:,t)
       LDT_ANNoutput_data%value(:,:,t) = temp_out(:,:,t)
    enddo
    ! scale the data
    do g=1,LDT_rc%ngrid(n)
       do p=1,LDT_ANNdata%nppb-1
          LDT_ANNdata%maxinp(g,p) = maxval(LDT_ANNinput_data%value(g,p,:))
          LDT_ANNdata%mininp(g,p) = minval(LDT_ANNinput_data%value(g,p,:))
       enddo
       LDT_ANNdata%maxout(g,1) = maxval(LDT_ANNoutput_data%value(g,1,:))
       LDT_ANNdata%minout(g,1) = minval(LDT_ANNoutput_data%value(g,1,:))
    enddo
    do t=1,LDT_ANNdata%ntimes
       do g=1,LDT_rc%ngrid(n)
          do p=1,LDT_ANNdata%ninputs
             if((LDT_ANNdata%maxinp(g,p)-LDT_ANNdata%mininp(g,p)).ne.0) then 
                LDT_ANNinput_data%value(g,p,t) = & 
                     ((LDT_ANNinput_data%value(g,p,t) - &
                     LDT_ANNdata%mininp(g,p))/&
                     (LDT_ANNdata%maxinp(g,p)-LDT_ANNdata%mininp(g,p)) - 0.5)*2
             endif
          enddo
          LDT_ANNoutput_data%value(g,1,t) = & 
                  ((LDT_ANNoutput_data%value(g,1,t) - &
                  LDT_ANNdata%minout(g,1))/&
                  (LDT_ANNdata%maxout(g,1)-LDT_ANNdata%minout(g,1)) - 0.5)*2
          
       enddo
    enddo
    write(LDT_logunit,*) 'Finished normalizing the data ..'
  end subroutine ANN_scaleData
!BOP
!
! !ROUTINE: LDT_logSingleANNdata
! \label{LDT_logSingleANNdata}
! 
! !INTERFACE:
  subroutine LDT_logSingleANNdata(n, value, pindex, iomode, name, units)
! !USES: 
    use LDT_coreMod, only     : LDT_rc, LDT_domain
    use LDT_logMod,  only     : LDT_logunit, LDT_endrun

    implicit none
! !ARGUMENTS: 
    integer, intent(in) :: n 
    integer             :: pindex
    real                :: value(LDT_rc%lnc(n), LDT_rc%lnr(n))
    integer             :: iomode
    character(len=*)    :: name
    character(len=*)    :: units
! 
! !DESCRIPTION: 
!  This subroutine maps the processed observations onto the LDT data
!  structures for future temporal averaging and comparisons. The 
!  data are also filtered using the specified external mask. 
!
!EOP
    integer :: k,i,c,r,gid

    if(iomode.eq.1) then 
       LDT_ANNinput_data%varName(pindex) = name
       LDT_ANNinput_data%varUnits(pindex) = units
    else
       LDT_ANNoutput_data%varName(pindex) = name
       LDT_ANNoutput_data%varUnits(pindex) = units
    endif

    do r=1,LDT_rc%lnr(n)
       do c=1,LDT_rc%lnc(n)
          if(value(c,r).ne.LDT_rc%udef) then 
             if(LDT_domain(n)%gindex(c,r).ne.-1) then 
                gid = LDT_domain(n)%gindex(c,r)
                if(iomode.eq.1) then 
                   LDT_ANNinput_data%value_hold(gid,pindex) = & 
                        LDT_ANNinput_data%value_hold(gid,pindex) + & 
                        value(c,r)
                   LDT_ANNinput_data%count(gid,pindex) = &
                        LDT_ANNinput_data%count(gid,pindex) + 1
                elseif(iomode.eq.2) then 
                   LDT_ANNoutput_data%value_hold(gid,1) = & 
                        LDT_ANNoutput_data%value_hold(gid,1) + & 
                        value(c,r)
                   LDT_ANNoutput_data%count(gid,1) = &
                        LDT_ANNoutput_data%count(gid,1) + 1
                endif
             endif
          endif
       enddo
    enddo
  end subroutine LDT_logSingleANNdata

end module LDT_ANNMod
