!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center Land Data Toolkit (LDT) v1.0
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
#include "LDT_misc.h"
module JULES43_parmsMod
!BOP
!
! !MODULE: JULES43_parmsMod
!
! !DESCRIPTION:
!  The code in this file implements routines to read JULES43 parameter
!  data. 
!  \subsubsection{Overview}
!  This routines in this module provides routines to read the 
!  JULES43 parameter file data.
!
! !REVISION HISTORY:
!
!  04 Nov 2013: K. Arsenault: Added layers for JULES43 model
!
#if(defined USE_NETCDF3 || defined USE_NETCDF4)
  use netcdf
#endif

  use ESMF
  use LDT_coreMod
  use LDT_historyMod
  use LDT_paramDataMod
  use LDT_logMod
  use LDT_paramMaskCheckMod

  implicit none

  PRIVATE
!------------------------------------------------------------------------------
! !PUBLIC MEMBER FUNCTIONS:
!------------------------------------------------------------------------------
  public :: JULES43parms_init    !allocates memory for required structures
  public :: JULES43parms_writeHeader
  public :: JULES43parms_writeData

!------------------------------------------------------------------------------
! !PUBLIC TYPES:
!------------------------------------------------------------------------------
  public :: JULES43_struc

  type, public :: jules_type_dec
     real      :: jules_undef

! -  JULES43-specific:
     type(LDT_paramEntry) :: jules  ! JULES43 parameters (collective)

  end type jules_type_dec

  type(jules_type_dec), allocatable :: JULES43_struc(:)

contains

!BOP
! 
! !ROUTINE: JULES43parms_init
! \label{JULES43parms_init}
! 
! !INTERFACE:
  subroutine JULES43parms_init

! !USES:
   use LDT_logMod,    only : LDT_verify, LDT_endrun, &
             LDT_getNextUnitNumber, LDT_releaseUnitNumber
!
! !DESCRIPTION:
!
! Joint UK Land Environment Simulator, v4.3 (JULES43) model parameters.
! 
!  The routines invoked are: 
!  \begin{description}
!   \item[julesParmssetup](\ref{julesParmssetup}) \newline
!    calls the registry to invoke the julesParms setup methods. 
!  \end{description}
!
!EOP
   implicit none
   integer  :: n
   integer  :: c,r,m,k
   integer  :: rc
   integer  :: file_status
   logical  :: file_exists
   logical  :: jules_select 

   ! _____________________________________________________________________

   allocate(JULES43_struc(LDT_rc%nnest))

   jules_select = .false.
   do n=1,LDT_rc%nnest
      ! - JULES43 parameters:
      call set_param_attribs(JULES43_struc(n)%jules,"JULES43")
      if( JULES43_struc(n)%jules%selectOpt == 1 ) then
         jules_select = .true.
      endif
   enddo

   if( jules_select ) then
     write(LDT_logunit,*)" - - - - - - - - - - JULES v4.3 LSM Parameters - - - - - - - - - - - - -"

     write(LDT_logunit,*)"[INFO] NO JULES v.43 Model Parameters available yet ..."
   endif


 end subroutine JULES43parms_init


 subroutine JULES43parms_writeHeader(n,ftn,dimID,monthID)
   
   integer   :: n 
   integer   :: ftn
   integer   :: dimID(3)
   integer   :: monthID
   
   integer   :: t_dimID(3)
   
   if( JULES43_struc(n)%jules%selectOpt == 1 ) then

   endif
   
   
 end subroutine JULES43parms_writeHeader
 
  subroutine JULES43parms_writeData(n,ftn)

    integer   :: n 
    integer   :: ftn

    if( JULES43_struc(n)%jules%selectOpt == 1 ) then
!      call JULES43parms_writeData(n,ftn)
!      call JULES43parms_writeData(n,ftn)
    endif

  end subroutine JULES43parms_writeData

!BOP
! !ROUTINE:  set_param_attribs
! \label{set_param_attribs}
!
! !INTERFACE:
  subroutine set_param_attribs(paramEntry, short_name)

! !DESCRIPTION:
!   This routine reads over the parameter attribute entries
!   in the param_attribs.txt file.
!
! !USES:
   type(LDT_paramEntry),intent(inout) :: paramEntry
   character(len=*),    intent(in)    :: short_name

! ____________________________________________________
    
   
   paramEntry%short_name = trim(short_name)
   paramEntry%vlevels = 1
   paramEntry%selectOpt = 1
   paramEntry%source = "JULES43"
   paramEntry%units ="none"
   paramEntry%num_times = 1
   paramEntry%num_bins = 1
   paramEntry%standard_name = trim(short_name)

  end subroutine set_param_attribs

end module JULES43_parmsMod

